@@ -1,4 +1,5 @@
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.Exts.Extension
@@ -39,7 +40,11 @@ module Language.Haskell.Exts.Extension (
 
     ) where
 
+#if __GLASGOW_HASKELL__ < 710
 import Control.Applicative ((<$>), (<|>))
+#else
+import Control.Applicative ((<|>))
+#endif
 import Data.Array (Array, accumArray, bounds, Ix(inRange), (!))
 import Data.List (nub)
 import Data.Maybe (fromMaybe)
@@ -519,6 +524,12 @@ data KnownExtension =
   -- start with an underscore as wildcards. For example, `foo :: _x` is
   -- equivalent to `foo :: _`.
   | NamedWildCards
+
+  | TypeApplications
+
+  | TypeFamilyDependencies
+
+  | OverloadedLabels
   deriving (Show, Read, Eq, Ord, Enum, Bounded, Data, Typeable)
 
 -- | Certain extensions imply other extensions, and this function
@@ -553,6 +564,7 @@ impliesExts = go
                     ExistentialQuantification -> [ExplicitForAll]
                     ImpredicativeTypes   -> [ExplicitForAll]
                     PolyKinds           -> [KindSignatures]
+                    TypeFamilyDependencies -> [TypeFamilies]
                     -- Deprecations
                     RecordPuns          -> [NamedFieldPuns]
                     PatternSignatures   -> [ScopedTypeVariables]
