@@ -36,6 +36,7 @@ import Control.Monad (when)
 
 data Token
         = VarId String
+        | LabelVarId String
         | QVarId (String,String)
         | IDupVarId (String)        -- duplicable implicit parameter
         | ILinVarId (String)        -- linear implicit parameter
@@ -88,6 +89,7 @@ data Token
         | LeftArrow
         | RightArrow
         | At
+        | TApp -- '@' but have to check for preceeding whitespace
         | Tilde
         | DoubleArrow
         | Minus
@@ -142,6 +144,7 @@ data Token
         | GENERATED
         | CORE
         | UNPACK
+        | NOUNPACK
         | OPTIONS (Maybe String,String)
 --        | CFILES  String
 --        | INCLUDE String
@@ -330,6 +333,7 @@ pragmas = [
  ( "generated",         GENERATED       ),
  ( "core",              CORE            ),
  ( "unpack",            UNPACK          ),
+ ( "nounpack",          NOUNPACK        ),
  ( "language",          LANGUAGE        ),
  ( "minimal",           MINIMAL         ),
  ( "no_overlap",        NO_OVERLAP      ),
@@ -347,6 +351,12 @@ isHSymbol c = c `elem` ":!#%&*./?@\\-" || ((isSymbol c || isPunctuation c) && no
 
 isPragmaChar c = isAlphaNum c || c == '_'
 
+-- Used in the lexing of type applications
+-- Why is it like this? I don't know exactly but this is how it is in
+-- GHC's parser.
+isOpSymbol :: Char -> Bool
+isOpSymbol c = c `elem` "!#$%&*+./<=>?@\\^|-~"
+
 -- | Checks whether the character would be legal in some position of a qvar.
 --   Means that '..' and "AAA" will pass the test.
 isPossiblyQvar :: Char -> Bool
@@ -734,6 +744,23 @@ lexStdToken = do
 
         ':':']':_ | ParallelArrays `elem` exts -> discard 2 >> return ParArrayRightSquare
 
+        -- Lexed seperately to deal with visible type applciation
+
+        '@':c:_ | TypeApplications `elem` exts
+                   -- Operator starting with an '@'
+                   && not (isOpSymbol c) -> do
+                                                lc <- getLastChar
+                                                if isIdent lc
+                                                  then discard 1 >> return At
+                                                  else discard 1 >> return TApp
+
+        '#':c:_ | OverloadedLabels `elem` exts
+                   && isLower c              -> do
+                                                  discard 1
+                                                  [ident] <- lexIdents
+                                                  return $ LabelVarId ident
+
+
         c:_ | isDigit c -> lexDecimalOrFloat
 
             | isUpper c -> lexConIdOrQual ""
@@ -801,8 +828,8 @@ lexStdToken = do
                         idents <- lexIdents
                         return $ ident : idents
                  '#':_ | MagicHash `elem` exts -> do
-                        discard 1
-                        return [ident ++ "#"]
+                        hashes <- lexWhile (== '#')
+                        return [ident ++ hashes]
                  _ -> return [ident]
 
             lexQuasiQuote :: Char -> Lex a Token
@@ -1247,6 +1274,7 @@ isBinDigit c =  c >= '0' && c <= '1'
 showToken :: Token -> String
 showToken t = case t of
   VarId s           -> s
+  LabelVarId s      -> '#':s
   QVarId (q,s)      -> q ++ '.':s
   IDupVarId s       -> '?':s
   ILinVarId s       -> '%':s
@@ -1293,6 +1321,7 @@ showToken t = case t of
   LeftArrow         -> "<-"
   RightArrow        -> "->"
   At                -> "@"
+  TApp              -> "@"
   Tilde             -> "~"
   DoubleArrow       -> "=>"
   Minus             -> "-"
@@ -1337,6 +1366,7 @@ showToken t = case t of
   GENERATED         -> "{-# GENERATED"
   CORE              -> "{-# CORE"
   UNPACK            -> "{-# UNPACK"
+  NOUNPACK          -> "{-# NOUNPACK"
   OPTIONS (mt,_)    -> "{-# OPTIONS" ++ maybe "" (':':) mt ++ " ..."
 --  CFILES  s         -> "{-# CFILES ..."
 --  INCLUDE s         -> "{-# INCLUDE ..."
