@@ -2,14 +2,15 @@
 {-# OPTIONS_HADDOCK hide #-}
 module Language.Haskell.Exts.ParseSyntax where
 
-import Language.Haskell.Exts.Annotated.Syntax hiding ( Type(..), Asst(..), Exp(..), FieldUpdate(..), XAttr(..), Context(..) )
-import qualified Language.Haskell.Exts.Annotated.Syntax as S ( Type(..), Promoted(..) )
+import Language.Haskell.Exts.Syntax hiding ( Type(..), Asst(..), Exp(..), FieldUpdate(..), XAttr(..), Context(..) )
+import qualified Language.Haskell.Exts.Syntax as S ( Type(..), Promoted(..) )
 
 ---------------------------------------
 -- Expressions as we parse them (and patterns, and regular patterns)
 
 data PExp l
     = Var l (QName l)                       -- ^ variable
+    | OverloadedLabel l String              -- ^ overloaded label #foo
     | IPVar l (IPName l)                    -- ^ implicit parameter variable
     | Con l (QName l)                       -- ^ data constructor
     | Lit l (Literal l)                     -- ^ literal constant
@@ -25,12 +26,10 @@ data PExp l
                                             --   the last statement in the list
                                             --   should be an expression.
     | MDo l [Stmt l]                        -- ^ @mdo@-expression
---    | Tuple [PExp]                        -- ^ tuple expression
     | TupleSection l Boxed [Maybe (PExp l)] -- ^ tuple section expression, e.g. @(,,3)@
     | List l [PExp l]                       -- ^ list expression
     | ParArray l [PExp l]                   -- ^ parallel array expression
     | Paren l (PExp l)                      -- ^ parenthesized expression
---     RightSection QOp PExp                -- ^ right section @(@/qop/ /exp/@)@
     | RecConstr l (QName l) [PFieldUpdate l]
                                             -- ^ record construction expression
     | RecUpdate l (PExp l) [PFieldUpdate l]
@@ -104,6 +103,7 @@ data PExp l
 
 -- LambdaCase
     | LCase l [Alt l]                       -- ^ @\case@ /alts/
+    | TypeApp l (S.Type l)
    deriving (Eq,Show,Functor)
 
 data PFieldUpdate l
@@ -118,6 +118,7 @@ data ParseXAttr l = XAttr l (XName l) (PExp l)
 instance Annotated PExp where
     ann e = case e of
         Var l _                 -> l
+        OverloadedLabel l _     -> l
         IPVar l _               -> l
         Con l _                 -> l
         Lit l _                 -> l
@@ -183,9 +184,11 @@ instance Annotated PExp where
 
         LCase l _               -> l
         MultiIf l _             -> l
+        TypeApp l _             -> l
 
     amap f e' = case e' of
         Var l qn                -> Var   (f l) qn
+        OverloadedLabel l qn    -> OverloadedLabel (f l) qn
         IPVar l ipn             -> IPVar (f l) ipn
         Con l qn                -> Con   (f l) qn
         Lit l lit               -> Lit   (f l) lit
@@ -251,6 +254,7 @@ instance Annotated PExp where
 
         LCase l alts -> LCase (f l) alts
         MultiIf l alts -> MultiIf (f l) alts
+        TypeApp l ty -> TypeApp (f l) ty
 
 instance Annotated PFieldUpdate where
     ann (FieldUpdate l _  _) = l
@@ -305,8 +309,9 @@ data PType l
      | TyKind  l (PType l) (Kind l)             -- ^ type with explicit kind signature
      | TyPromoted l (S.Promoted l)              -- ^ promoted data type
      | TySplice l (Splice l)                    -- ^ template haskell splice type
-     | TyBang l (BangType l) (PType l)          -- ^ Strict type marked with \"@!@\" or type marked with UNPACK pragma.
+     | TyBang l (BangType l) (Unpackedness l) (PType l) -- ^ Strict type marked with \"@!@\" or type marked with UNPACK pragma.
      | TyWildCard l (Maybe (Name l))            -- ^ Type wildcard
+     | TyQuasiQuote l String String             -- ^ @[qq| |]@
   deriving (Eq, Show, Functor)
 
 instance Annotated PType where
@@ -325,8 +330,9 @@ instance Annotated PType where
       TyPromoted l   _              -> l
       TyPred l _                    -> l
       TySplice l _                  -> l
-      TyBang  l _ _                 -> l
+      TyBang  l _ _ _                 -> l
       TyWildCard  l _               -> l
+      TyQuasiQuote l _ _            -> l
     amap f t' = case t' of
       TyForall l mtvs mcx t         -> TyForall (f l) mtvs mcx t
       TyFun   l t1 t2               -> TyFun (f l) t1 t2
@@ -342,8 +348,9 @@ instance Annotated PType where
       TyPromoted l   p              -> TyPromoted (f l)   p
       TyPred l asst                 -> TyPred (f l) asst
       TySplice l s                  -> TySplice (f l) s
-      TyBang  l b t                 -> TyBang (f l) b t
+      TyBang  l b u t                 -> TyBang (f l) b u t
       TyWildCard l mn               -> TyWildCard (f l) mn
+      TyQuasiQuote l n s            -> TyQuasiQuote (f l) n s
 
 data PAsst l
     = ClassA l (QName l) [PType l]
@@ -372,13 +379,3 @@ instance Annotated PAsst where
         EqualP l t1 t2      -> EqualP (f l) t1 t2
         ParenA l a          -> ParenA (f l) a
         WildCardA l mn      -> WildCardA (f l) mn
-
-
-unit_tycon, fun_tycon, list_tycon, unboxed_singleton_tycon :: l -> PType l
-unit_tycon              l = TyCon l (unit_tycon_name l)
-fun_tycon               l = TyCon l (fun_tycon_name l)
-list_tycon              l = TyCon l (list_tycon_name l)
-unboxed_singleton_tycon l = TyCon l (unboxed_singleton_tycon_name l)
-
-tuple_tycon :: l -> Boxed -> Int -> PType l
-tuple_tycon l b i         = TyCon l (tuple_tycon_name l b i)
